@@ -18,6 +18,7 @@ import Network.TLS.Struct
 import Network.TLS.Cipher
 import Network.TLS.Compression
 import Network.TLS.Packet
+import Network.TLS.ErrT
 import Network.TLS.Extension
 import Network.TLS.IO
 import Network.TLS.State hiding (getNegotiatedProtocol)
@@ -33,7 +34,6 @@ import Data.ByteString.Char8 ()
 
 import Control.Applicative ((<$>), (<*>))
 import Control.Monad.State
-import Control.Monad.Error
 import Control.Exception (SomeException)
 
 import Network.TLS.Handshake.Common
@@ -63,7 +63,18 @@ handshakeClient cparams ctx = do
     handshakeTerminate ctx
   where ciphers      = ctxCiphers ctx
         compressions = supportedCompressions $ ctxSupported ctx
-        getExtensions = sequence [sniExtension,secureReneg,npnExtention] >>= return . catMaybes
+        getExtensions = sequence [sniExtension
+                                 ,secureReneg
+                                 ,npnExtention
+                                 ,alpnExtension
+                                 {-
+                                 ,curveExtension,
+                                 ,ecPointExtension
+                                 ,sessionTicketExtension
+                                 ,heartbeatExtension
+                                 -}
+                                 ,signatureAlgExtension
+                                 ]
 
         toExtensionRaw :: Extension e => e -> ExtensionRaw
         toExtensionRaw ext = (extensionID ext, extensionEncode ext)
@@ -75,14 +86,33 @@ handshakeClient cparams ctx = do
         npnExtention = if isJust $ onNPNServerSuggest $ clientHooks cparams
                          then return $ Just $ toExtensionRaw $ NextProtocolNegotiation []
                          else return Nothing
+        alpnExtension = do
+            mprotos <- onSuggestALPN $ clientHooks cparams
+            case mprotos of
+                Nothing -> return Nothing
+                Just protos -> do
+                    usingState_ ctx $ setClientALPNSuggest protos
+                    return $ Just $ toExtensionRaw $ ApplicationLayerProtocolNegotiation protos
         sniExtension = if clientUseServerNameIndication cparams
                          then return $ Just $ toExtensionRaw $ ServerName [ServerNameHostName $ fst $ clientServerIdentification cparams]
                          else return Nothing
+
+        {-
+        curveExtension = return $ Just $ toExtensionRaw $ EllipticCurvesSupported [NamedCurve_secp256k1,NamedCurve_secp256r1]
+        ecPointExtension = return $ Just $ toExtensionRaw $ EcPointFormatsSupported [EcPointFormat_Uncompressed,EcPointFormat_AnsiX962_compressed_prime,EcPointFormat_AnsiX962_compressed_char2]
+
+        sessionTicketExtension = return $ Just $ toExtensionRaw $ SessionTicket
+
+        heartbeatExtension = return $ Just $ toExtensionRaw $ HeartBeat $ HeartBeat_PeerAllowedToSend
+        -}
+
+        signatureAlgExtension = return $ Just $ toExtensionRaw $ SignatureAlgorithms $ supportedHashSignatures $ clientSupported cparams
+
         sendClientHello = do
             crand <- getStateRNG ctx 32 >>= return . ClientRandom
             let clientSession = Session . maybe Nothing (Just . fst) $ clientWantSessionResume cparams
                 highestVer = maximum $ supportedVersions $ ctxSupported ctx
-            extensions <- getExtensions
+            extensions <- catMaybes <$> getExtensions
             startHandshake ctx highestVer crand
             usingState_ ctx $ setVersionIfUnset highestVer
             sendPacket ctx $ Handshake
@@ -174,6 +204,7 @@ sendClientData cparams ctx = sendCertificate >> sendClientKeyXchg >> sendCertifi
                     return $ CKX_RSA encryptedPreMaster
                 CipherKeyExchange_DHE_RSA -> getCKX_DHE
                 CipherKeyExchange_DHE_DSS -> getCKX_DHE
+                CipherKeyExchange_ECDHE_RSA -> getCKX_ECDHE
                 _ -> throwCore $ Error_Protocol ("client key exchange unsupported type", True, HandshakeFailure)
             sendPacket ctx $ Handshake [ClientKeyXchg ckx]
           where getCKX_DHE = do
@@ -186,6 +217,17 @@ sendClientData cparams ctx = sendCertificate >> sendClientKeyXchg >> sendCertifi
 
                     return $ CKX_DH clientDHPub
 
+                getCKX_ECDHE = do
+                    xver <- usingState_ ctx getVersion
+                    (ServerECDHParams ecdhparams serverECDHPub) <- fromJust <$> usingHState ctx (gets hstServerECDHParams)
+                    (clientECDHPriv, clientECDHPub) <- generateECDHE ctx ecdhparams
+
+                    case ecdhGetShared ecdhparams clientECDHPriv serverECDHPub of
+                        Nothing        -> throwCore $ Error_Protocol ("invalid server public key", True, HandshakeFailure)
+                        Just premaster -> do
+                            usingHState ctx $ setMasterSecretFromPre xver ClientRole premaster
+                            return $ CKX_ECDH clientECDHPub
+
         -- In order to send a proper certificate verify message,
         -- we have to do the following:
         --
@@ -275,10 +317,22 @@ onServerHello ctx cparams sentExts (ServerHello rver serverRan serverSession cip
         setVersion rver
     usingHState ctx $ setServerHelloParameters rver serverRan cipherAlg compressAlg
 
+    case extensionDecode False `fmap` (lookup extensionID_ApplicationLayerProtocolNegotiation exts) of
+        Just (Just (ApplicationLayerProtocolNegotiation [proto])) -> usingState_ ctx $ do
+            mprotos <- getClientALPNSuggest
+            case mprotos of
+                Just protos -> when (elem proto protos) $ do
+                    setExtensionALPN True
+                    setNegotiatedProtocol proto
+                _ -> return ()
+        _ -> return ()
+
     case extensionDecode False `fmap` (lookup extensionID_NextProtocolNegotiation exts) of
         Just (Just (NextProtocolNegotiation protos)) -> usingState_ ctx $ do
-            setExtensionNPN True
-            setServerNextProtocolSuggest protos
+            alpnDone <- getExtensionALPN
+            unless alpnDone $ do
+                setExtensionNPN True
+                setServerNextProtocolSuggest protos
         _ -> return ()
 
     case resumingSession of
@@ -324,6 +378,8 @@ processServerKeyExchange ctx (ServerKeyXchg origSkx) = do
                     doDHESignature dhparams signature SignatureRSA
                 (CipherKeyExchange_DHE_DSS, SKX_DHE_DSS dhparams signature) -> do
                     doDHESignature dhparams signature SignatureDSS
+                (CipherKeyExchange_ECDHE_RSA, SKX_ECDHE_RSA ecdhparams signature) -> do
+                    doECDHESignature ecdhparams signature SignatureRSA
                 (cke, SKX_Unparsed bytes) -> do
                     ver <- usingState_ ctx getVersion
                     case decodeReallyServerKeyXchgAlgorithmData ver cke bytes of
@@ -338,6 +394,13 @@ processServerKeyExchange ctx (ServerKeyXchg origSkx) = do
             when (not verified) $ throwCore $ Error_Protocol ("bad " ++ show signatureType ++ " for dhparams", True, HandshakeFailure)
             usingHState ctx $ setServerDHParams dhparams
 
+        doECDHESignature ecdhparams signature signatureType = do
+            -- TODO verify DHParams
+            expectedData <- generateSignedECDHParams ctx ecdhparams
+            verified <- signatureVerify ctx signatureType expectedData signature
+            when (not verified) $ throwCore $ Error_Protocol ("bad " ++ show signatureType ++ " for dhparams", True, HandshakeFailure)
+            usingHState ctx $ setServerECDHParams ecdhparams
+
 processServerKeyExchange ctx p = processCertificateRequest ctx p
 
 processCertificateRequest :: Context -> Handshake -> IO (RecvState IO)
